#!/bin/sh 
#
# Usage: addfilter [-v] device capacity handle [-src ipaddr] [-dst ipaddr] 
#   where ipaddr = A.B.C.D/netmask or A.B.C.D[:port]
#            netmask = number of 1 bits in the mask
#            e.g., 255.255.255.0 is 24; 255.255.255.240 is 28
#
# Adds a filter to an already created flowid  
#
#
# Author:  Marc Siriratsivawong
# Version: $Revision: 1.2 $, $Date: 2001/08/17 15:08:36 $
# $Id: addfilter.cgi,v 1.2 2001/08/17 15:08:36 jfhuard Exp $
#

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo

echo Content-type: text/plain
echo

usage() {
    echo "usage: $0 [-v] -dev device -rhdl rootHandle "
    echo "               -phdl parentHandle -chdl classHandle "
    echo "               -prio classPriority "
    echo "               -dir src|dst -addr ipaddr"
    echo "       where ipaddr = A.B.C.D/netmask or A.B.C.D[:port]"
    echo "       netmask = number of 1 bits in the mask"
    echo "       e.g., 255.255.255.0 is 24; 255.255.255.240 is 28"
    echo ""
}

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) echo ERROR ; usage ; exit 1;;
14)
	while [ $# -gt 0 ] 
	    do
		case $1 in
		    -dev)  DEVICE=$2       ;;
		    -rhdl) ROOTHANDLE=$2   ;; #root handle
		    -phdl) PARHANDLE=$2    ;; #Parent Handle
                    -chdl) CLSHANDLE=$2    ;; #class handle
		    -prio) CLSPRIO=$2      ;; #class prio
                    -dir)  DIR=$2          ;; #filter direction
                    -addr) ADDR=$2         ;; #filter address
                    *) echo ERROR wrong argument $1 ; exit 1;;
		esac
		shift 2
	    done ;;

*)  echo ERROR ; usage ; exit 1;;
esac

if [ $VERBOSE -eq 1 ]; then 
    echo interface:     $DEVICE
    echo root handle:   $ROOTHANDLE
    echo parent handle: $PARHANDLE
    echo class handle:  $CLSHANDLE
    echo priority:      $CLSPRIO
    echo direction:     $DIR
    echo address:       $ADDR
    echo 
fi

CHDL="$ROOTHANDLE":"$CLSHANDLE"

RET=`$TC filter add dev $DEVICE parent $ROOTHANDLE: protocol ip \
    prio $CLSPRIO u32 match ip $DIR $ADDR flowid $CHDL 2>&1` 

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot create filter
    exit 1
fi

if [ $VERBOSE -eq 1 ] ; then 
    $TC class show dev $DEVICE | grep $CLSHANDLE 
fi

echo SUCCESS $DEVICE $ROOTHANDLE $ROOTRATE $PARHANDLE $CLSHANDLE  $CLSRATE $CLSPRIO $CLSBUFF $DIR $ADDR
    
exit


#
# $Log: addfilter.cgi,v $
# Revision 1.2  2001/08/17 15:08:36  jfhuard
# removed absolute path for tc
#
# Revision 1.1  2001/08/17 15:06:09  marcs
# *** empty log message ***
#
#
#