#!/bin/sh 
USAGE="usage: $0 [-hv] -dev device"
#
# Make an allocation by src/dst IP, by attaching filter to ingress 
# qdisc parent.
#
# Author:  nemo@invisiblehand.net
# Version: $Revision: 1.2 $, $Date: 2005/02/01 02:02:31 $
# $Id: alloc-ingress.cgi,v 1.2 2005/02/01 02:02:31 nemo Exp $

source routux.conf

echo Content-type: text/plain
echo

usage() {
    echo "usage: $0 [-v] -dev device -flowid flowid "
    echo "               -rate classRate -prio classPriority "
    echo "               -buf classBuffSize -dir src|dst -addr ipaddr"
    echo "       where ipaddr = A.B.C.D/netmask or A.B.C.D[:port]"
    echo "       netmask = number of 1 bits in the mask"
    echo "       e.g., 255.255.255.0 is 24; 255.255.255.240 is 28"
    echo "       unit for rate in kbit/s and for buffer in kB"
    echo ""
}

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) echo ERROR ; usage ; exit 1;;
14)
	while [ $# -gt 0 ] 
	    do
		case $1 in
		    -dev)  DEVICE=$2       ;;
                    -flowid) FLOWID=$2    ;; #flow id (aka class handle)
		    -rate) CLSRATE=$2      ;; #class rate
		    -prio) CLSPRIO=$2      ;; #class prio
                    -buf)  CLSBUFF=$2      ;; #class queue size
                    -dir)  DIR=$2          ;; #src or dst
                    -addr) ADDR=$2         ;; #filter address
                    *) echo ERROR wrong argument $1 ; exit 1;;
		esac
		shift 2
	    done ;;

*)  echo ERROR ; usage ; exit 1;;
esac

if [ $VERBOSE -eq 1 ]; then 
    echo interface:     $DEVICE
    echo flowid:        $FLOWID
    echo class rate:    $CLSRATE
    echo priority:      $CLSPRIO
    echo buffer:        $CLSBUFF
    echo direction:     $DIR
    echo address:       $ADDR
    echo 
fi

# ingress qdisc is classless, parent is always ffff:

RET=`$TC filter add dev $DEVICE parent ffff: protocol ip prio $CLSPRIO u32 match ip $DIR $ADDR flowid :"$FLOWID" police rate "$CLSRATE"kbit mtu $MTU burst "$CLSBUFF"K  drop 2>&1`

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot create filter: $RET
    exit 1
fi

echo SUCCESS $DEVICE flowid=$FLOWID  prio=$CLSPRIO  $DIR $ADDR rate=$CLSRATE buf=$CLSBUFF
    
exit
