#!/bin/sh 
#
# Usage: alloc [-v] device capacity handle [-src ipaddr] [-dst ipaddr] 
#   where ipaddr = A.B.C.D/netmask or A.B.C.D[:port]
#            netmask = number of 1 bits in the mask
#            e.g., 255.255.255.0 is 24; 255.255.255.240 is 28
#
# Initialize the specified device with capacity and
# queueing discipline CBQ.  
#
# Note: this script is CBQ protocol dependent
#       assumes that the line speed is 100Mbit
#
# Author:  JF Huard
# Version: $Revision: 1.7 $, $Date: 2001/04/17 22:24:56 $
# $Id: alloc2.cgi,v 1.7 2001/04/17 22:24:56 marcs Exp $
#

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo

echo Content-type: text/plain
echo

usage() {
    echo "usage: $0 [-v] -dev device -ls lineRate -rhdl rootHandle "
    echo "               -phdl parentHandle -chdl classHandle "
    echo "               -rate classRate -prio classPriority "
    echo "               -buf classBuffSize -dir src|dst -addr ipaddr"
    echo "       where ipaddr = A.B.C.D/netmask or A.B.C.D[:port]"
    echo "       netmask = number of 1 bits in the mask"
    echo "       e.g., 255.255.255.0 is 24; 255.255.255.240 is 28"
    echo "       unit for rate in kbit/s and for buffer in kB"
    echo ""
}

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) echo ERROR ; usage ; exit 1;;
20)
	while [ $# -gt 0 ] 
	    do
		case $1 in
		    -dev)  DEVICE=$2       ;;
		    -rhdl) ROOTHANDLE=$2   ;; #root handle
		    -ls)   ROOTRATE=$2     ;; #root rate (linespeed)
		    -phdl) PARHANDLE=$2    ;; #Parent Handle
                    -chdl) CLSHANDLE=$2    ;; #class handle
		    -rate) CLSRATE=$2      ;; #class rate
		    -prio) CLSPRIO=$2      ;; #class prio
                    -buf)  CLSBUFF=$2      ;; #class queue size
                    -dir)  DIR=$2          ;; #filter direction
                    -addr) ADDR=$2         ;; #filter address
                    *) echo ERROR wrong argument $1 ; exit 1;;
		esac
		shift 2
	    done ;;

*)  echo ERROR ; usage ; exit 1;;
esac

if [ $VERBOSE -eq 1 ]; then 
    echo interface:     $DEVICE
    echo root handle:   $ROOTHANDLE
    echo line rate:     $ROOTRATE
    echo parent handle: $PARHANDLE
    echo class handle:  $CLSHANDLE
    echo class rate:    $CLSRATE
    echo priority:      $CLSPRIO
    echo buffer:        $CLSBUFF
    echo direction:     $DIR
    echo address:       $ADDR
    echo 
fi


QDISC_RSP=`$TC qdisc show dev $DEVICE 2>&1`
NODEV=`$ECHO $QDISC_RSP | $GREP "Cannot find device"`
if [ -n "$NODEV" ]; then
    echo ERROR $DEVICE not found
    exit 1
else
    if [ -z "$QDISC_RSP" ] ; then
	echo ERROR $DEVICE not initialized
	exit 1
    fi
fi

ROOTLN=`$TC class show dev $DEVICE | $GREP root | $GREP cbq`

if [ -z "$ROOTLN" ] ; then
    echo ERROR $DEVICE CBQ root class not initialized
    exit 1
else
    LINESPEED=`$ECHO $ROOTLN | $AWK '{ print $6 }'`
#    ROOTHANDLE=`$ECHO $ROOTLN | $AWK '{ print $3 }'`
    if [ -z "$LINESPEED" ] ; then
        echo ERROR cannot extract device line speed
	exit 1;
    fi
fi

CLSHANDLE=$CLSHANDLE

if [ $VERBOSE -eq 1 ] ; then 
    echo root   handle: $ROOTHANDLE
    echo parent handle: $ROOTHANDLE:$PARHANDLE
    echo class  handle: $ROOTHANDLE:$CLSHANDLE
    echo line speed:    $LINESPEED
fi

#set for the first time a class/tbf/filter 

RHDL="$ROOTHANDLE":"$PARHANDLE"
CHDL="$ROOTHANDLE":"$CLSHANDLE"

RET=`$TC class add dev $DEVICE parent $RHDL classid $CHDL  \
    cbq bandwidth "$ROOTRATE"kbit rate "$CLSRATE"kbit cell 8 weight 1Mbit \
    prio 8  maxburst 20 avpkt 1000 2>&1`

if [ -n "$RET" ] ; then
   echo ERROR $DEVICE cannot add class
   exit 1
fi

RET=`$TC qdisc  add dev $DEVICE parent $CHDL tbf rate "$CLSRATE"kbit \
    buffer "$CLSBUFF"K limit "$CLSBUFF"K 2>&1`

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot add tbf queue
    exit 1
fi

RET=`$TC filter add dev $DEVICE parent $ROOTHANDLE: protocol ip \
    prio $CLSPRIO u32 match ip $DIR $ADDR flowid $CHDL 2>&1` 

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot create filter
    exit 1
fi

if [ $VERBOSE -eq 1 ] ; then 
    $TC class show dev $DEVICE | grep $CLSHANDLE 
fi

echo SUCCESS $DEVICE $ROOTHANDLE $ROOTRATE $PARHANDLE $CLSHANDLE  $CLSRATE $CLSPRIO $CLSBUFF $DIR $ADDR
    
exit


#
# $Log: alloc2.cgi,v $
# Revision 1.7  2001/04/17 22:24:56  marcs
# added parse support for prefix of command line parameters
#
# Revision 1.6  2001/01/13 04:06:19  jfhuard
# corrected the weight parameter in class creation
#
# Revision 1.5  2000/12/19 20:17:03  jfhuard
# filter line parent handle fixed - not RHDL, but ROOTHANDLE
#
# Revision 1.4  2000/12/18 20:54:23  jfhuard
# typo -- forgotten 'K' for limit on line 135
#
# Revision 1.3  2000/12/18 19:25:25  jfhuard
# *** empty log message ***
#
# Revision 1.2  2000/12/18 19:24:51  jfhuard
# changed the parent handle in
#
# Revision 1.1  2000/12/02 03:45:05  jfhuard
# new allocation script has a new api compared to alloc.cgi
# not backward compatible
#
#