#!/bin/sh 
#
# Usage: clear [-v] device
#
# Reset the specified device to its initial CBQ state 
# by deleting all the filters and associated classes.
# In particular, it reset the device with its line rate
# (root rate) and capacity (total controlled resource).
#
# Note: this script is CBQ protocol dependent
#
# Author:  JF Huard
# Version: $Revision: 1.5 $, $Date: 2001/05/11 00:58:39 $
# $Id: clear2.cgi,v 1.5 2001/05/11 00:58:39 marcs Exp $

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo
INIT=CGI_DIR/init2.cgi


echo Content-type: text/plain
echo

usage() {
    echo "usage: $0 [-v] -dev device -rhdl rootHandle -shdl sellerHandle"
    echo ""
}

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) echo ERROR ; usage ; exit 1;;
6)
	while [ $# -gt 0 ] 
	    do
		case $1 in
		    -dev)  DEVICE=$2       ;;
		    -rhdl) ROOTHDL=$2      ;; #root handle
		    -shdl) SELLHDL=$2      ;; #seller handle
                    *) echo ERROR wrong argument $1 ; exit 1;;
		esac
		shift 2
	    done ;;
*) echo ERROR ; usage ; exit 1;;
esac

if [ $VERBOSE -eq 1 ] ; then 
    echo interface: $DEVICE
fi

QDISC_RSP=`$TC qdisc show dev $DEVICE 2>&1`
NODEV=`$ECHO $QDISC_RSP | $GREP "Cannot find device"`
if [ -n "$NODEV" ]; then
    echo ERROR $DEVICE not found
    exit 1
else
    if [ -z "$QDISC_RSP" ] ; then
	echo ERROR $DEVICE not initialized - cannot clear
	exit 1
    fi
fi

CLASS_RSP=`$TC class show dev $DEVICE | $GREP root | $GREP cbq`

if [ -z "$CLASS_RSP" ] ; then
    echo ERROR $DEVICE CBQ root class not initialized
    exit 1
else
    ROOTRATE=`echo $CLASS_RSP | $AWK 'BEGIN{RS="class"}  match($3,"root") { print $5}' | sed -e "s%Kbit%%g"`
    ROOTHANDLE=`echo $CLASS_RSP | $AWK 'BEGIN{RS="class"} \
                                    { split($2, beh, ":"); print beh[1]}'`
fi

GREPLINE="class cbq $ROOTHANDLE:$SELLHDL parent $ROOTHANDLE: rate"
RESLINE=`$TC class show dev $DEVICE | $GREP "$GREPLINE"`

if [ -n "$RESLINE" ] ; then
    CAPRATE=`$ECHO $RESLINE | $AWK '{ print $7 }' | sed -e "s%Kbit%%g"`
else
   echo ERROR cannot get current capacity allocation
fi

if [ $VERBOSE -eq 1 ] ; then 
    echo root handle:    $ROOTHANDLE
    echo line speed:     $ROOTRATE
    echo seller handle:  $SELLHDL
    echo seller cap:     $CAPRATE
    exec $INIT -dev $DEVICE \
               -rhdl $ROOTHANDLE \
               -ls $ROOTRATE \
	       -shdl $SELLHDL \
	       -sr $CAPRATE
else
    exec $INIT -dev $DEVICE \
               -rhdl $ROOTHANDLE \
               -ls $ROOTRATE \
	       -shdl $SELLHDL \
	       -sr $CAPRATE 
fi

exit

#
# $Log: clear2.cgi,v $
# Revision 1.5  2001/05/11 00:58:39  marcs
# init variable was set incorrectly
#
# Revision 1.4  2001/04/17 22:24:56  marcs
# added parse support for prefix of command line parameters
#
# Revision 1.3  2000/12/02 07:26:09  nemo
# strip units from ROOTRATE and CAPRATE
#
# Revision 1.2  2000/12/02 03:58:50  jfhuard
# *** empty log message ***
#
#