#!/bin/sh 
USAGE="usage: $0 [-hv] device -rh rootHandle (def 1) -ls speed (def 100Mbit)\
 -sh sellerHandle (def 1) -sr sellerCapacity (def 1536Kbit)"
#
#
# Initialize the specified device with capacity and
# queueing discipline CBQ.  
#
# Note: - this script assumes CBQ scheduling mechanism and
#         assumes that the line speed is 100Mbit
#       - all rates are in kbit/sec
#       - buffer size in kB
#
# Author:  JF Huard
# Version: $Revision: 1.3 $, $Date: 2001/04/25 19:35:33 $
# $Id: init2.cgi,v 1.3 2001/04/25 19:35:33 marcs Exp $

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo
RESET=CGI_DIR/reset2.cgi

case $1 in
-r) shift ;;
*)
echo Content-type: text/plain
echo
esac


usage() {
    echo "usage: $0 [-hv] -dev device -rhdl rootHandle (def 1) -ls rate (def 100Mbit)"
    echo "        -shdl sellerHandle (def 1) -sr sellerCapacity (def 1536Kbit"
    echo ""
}


case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ default values ############

DEVICE=eth0
CAPACITY=1536
HANDLE=1
PARHANDLE=1
RATE=100000

############ parse command line parameters ############

case $# in
0) echo ERROR ; usage ; exit 1;;
2) DEVICE=$2 ;;
10) 
	while [ $# -gt 0 ] 
	    do
		case $1 in
		    -dev)   DEVICE=$2   ;;
		    -rhdl)  HANDLE=$2   ;;
		    -ls)    RATE=$2     ;;
		    -shdl)  PARHANDLE=$2;;
		    -sr)    CAPACITY=$2 ;;
                    *) echo ERROR wrong argument $1 ; exit 1;;
		esac
		shift 2
	    done ;;

*) echo ERROR ; usage ; exit 1;;
esac

if [ $VERBOSE -eq 1 ]; then 
    echo interface:     $DEVICE
    echo root handle:   $HANDLE
    echo line rate:     $RATE
    echo seller handle: $PARHANDLE
    echo seller cap:    $CAPACITY
    echo 
fi

############ Reset interface #############

RET=`$RESET -dev $DEVICE | $GREP SUCCESS`

if [ $VERBOSE -eq 1 ]; then 
    echo call to $RESET returned $RET
fi

if [ -z "$RET" ] ; then
    echo ERROR $DEVICE cannot reset device prior to initialization.
    exit 1
fi

############ Initialize root class ############
# root handle is $HANDLE:0

RET=`$TC qdisc add dev $DEVICE root handle $HANDLE:0 cbq \
          bandwidth "$RATE"kbit cell 8 avpkt 1000 mpu 64 2>&1`

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot create qdisc
    exit 1
fi

RET=`$TC class add dev $DEVICE parent $HANDLE:0 \
          classid $HANDLE:$PARHANDLE cbq bandwidth "$RATE"kbit \
          rate "$CAPACITY"kbit cell 8 weight 1Mbit prio 8 maxburst 20 avpkt 1000 2>&1`

if [ -n "$RET" ] ; then
    echo ERROR $DEVICE cannot set capacity for class
    exit 1
fi


############ return ############

if [ $VERBOSE -eq 1 ] ; then 
    $TC qdisc show dev $DEVICE
    $TC class show dev $DEVICE
    $TC filter show dev $DEVICE
fi

echo SUCCESS $DEVICE $RATE $HANDLE $CAPACITY

exit

#
# $Log: init2.cgi,v $
# Revision 1.3  2001/04/25 19:35:33  marcs
# fixed bug where reset.cgi was being called instead of reset2.cgi
#
# Revision 1.2  2001/04/17 22:24:56  marcs
# added parse support for prefix of command line parameters
#
# Revision 1.1  2000/12/02 03:46:53  jfhuard
# new api to init.cgi
# not backward compatible with init
#
#