#!/bin/sh 
#
# Usage: reset [-v] [device [handle]]
#
# Reset the router by removing the root queueing 
# discipline, classes and filters.  If optional device 
# and handle are specified, it removes the specified 
# queue, filter and class.
#
# Author:  JF Huard
# Version: $Revision: 1.3 $, $Date: 2001/10/09 21:23:55 $
# $Id: reset2.cgi,v 1.3 2001/10/09 21:23:55 jfhuard Exp $

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo
MON=CGI_DIR/mon.sh

usage() {
    echo "usage: $0  [-v] [-dev device [handle]]"
    echo ""
}

echo Content-type: text/plain
echo

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)  VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) DEVICE="" ; HANDLE="" ;;
2) DEVICE=$2 ; HANDLE="" ;;
3) DEVICE=$2 ; HANDLE=$3 ;;
*) echo ERROR ; usage ; exit 1 ;;
esac

if [ $VERBOSE -eq 1 ] ; then 
    echo interface: $DEVICE
    echo handle:    $HANDLE
fi

if [ -n "$DEVICE" ] ; then

    # we need to delete a specific device, check if its active

    QDISC_RSP=`$TC qdisc show dev $DEVICE 2>&1`
    NODEV=`$ECHO $QDISC_RSP | $GREP "Cannot find device"`
    if [ -z "$NODEV" ]; then
	if [ -z "$QDISC_RSP" ] ; then
	    echo SUCCESS $DEVICE
	    exit 1
	fi
    else
	echo ERROR $DEVICE not found
	exit 1
    fi

    ROOTLN=`$TC class show dev $DEVICE | $GREP root`

    if [ -z "$ROOTLN" ] ; then
	echo ERROR $DEVICE root class not initialized
	exit 1
    fi
    
    CURHANDLE=`$ECHO $ROOTLN | $AWK '{ print $3 }'`
    
    if [ -n "$HANDLE" ] ; then
	# check to make sure the fully specified queue/class is there
	# else we take whatever the current handle is.
	if [  "$HANDLE:" != "$CURHANDLE" ] ; then
	    echo ERROR $DEVICE $HANDLE class not initialized
	    exit 1 
	fi
    fi

    if [ $VERBOSE -eq 1 ] ; then 
	echo root handle for device: $CURHANDLE
	echo
    fi
    if [ -s $MON ] ; then

	# we need to list all the flows so that mon.sh can be invoked on them
	$TC filter show dev $DEVICE | \
	grep match | grep -v 0.0.0.0 | \
	$AWK -v mon=$MON -v device=$DEVICE 'BEGIN{line = 0;}{ \
	    line = line + 1; \
	    if (line != 1) { \
		split($3, root, ":"); \
		split($19, parcls, ":"); \
		cmdline = sprintf("/bin/sh %s %s %s %s %s", \
		mon, device, root[1], parcls[1], parcls[2]); \
		system(cmdline); \
	    } \
	}'
    fi

    # set the device list to be reset
    DEV_LIST=$DEVICE

else # get the list of all devices

    for i in 0 1 2 3 4 5 6 7
    do
	DEVICE=eth$i

	# we need to delete a specific device, check if its active

	DEVNOK=`$TC qdisc show dev $DEVICE 2>&1 | $GREP -v "Cannot find device"`

	if [ -n "$DEVNOK" ] ; then
	    ROOTLN=`$TC class show dev $DEVICE | $GREP root`

	    if [ -n "$ROOTLN" ] ; then
		OLD_LIST="$DEV_LIST"
		DEV_LIST="$OLD_LIST $DEVICE"
	    fi
	elif [ $VERBOSE -eq 1 ] ; then
	    echo $DEVICE absent
	fi
    done
fi

for i in $DEV_LIST 
do
    RET=`$TC qdisc del dev $i root 2>&1`

    if [ -n "$RET" ] ; then
	echo ERROR $i resetting device/interface
    else #if [ $VERBOSE -eq 1 ] ; then 
	echo SUCCESS $i
    fi
done

if [ -z "$DEV_LIST" ] ; then
    echo SUCCESS
fi
exit

#
# $Log: reset2.cgi,v $
# Revision 1.3  2001/10/09 21:23:55  jfhuard
# now execute mon.sh before resetting flows
#
# Revision 1.2  2001/04/17 22:24:56  marcs
# added parse support for prefix of command line parameters
#
# Revision 1.1  2000/12/02 03:48:00  jfhuard
# new api to reset
# not backward compatible
#
#