#!/bin/sh 
#
# Usage: show [-v] [device]
#
# Show the queueing discipline, classes and filters
# for a all or a given device.  
#
# Author:  JF Huard
# Version: $Revision: 1.5 $, $Date: 2001/09/17 14:25:56 $

TC=TC_DIR/tc
AWK=/bin/awk
GREP="/bin/grep -w"
ECHO=/bin/echo

usage() {
    echo "usage: $0 [-v] [-dev device]"
    echo ""
}

echo Content-type: text/plain
echo

case $1 in
-hv|-h|-vh) usage ; exit 1 ;;
-v*) VERBOSE=1 ; echo Verbose ON ; shift  ;;
*)   VERBOSE=0 ;;
esac

############ parse command line parameters ############

case $# in
0) DEVICE="" ;;
2) DEVICE=$2 ;;
*) echo ERROR ; usage ; exit 1;;
esac

# print the uptime of the system
echo -n "uptime "
cat /proc/uptime

if [ $VERBOSE -eq 1 ] ; then 
    if [ -z "$DEVICE" ] ; then
	echo interface: ALL
    else
	echo interface: $DEVICE
    fi
fi

if [ -n "$DEVICE" ] ; then

    QDISC_RSP=`$TC qdisc show dev $DEVICE 2>&1`
    NODEV=`$ECHO $QDISC_RSP | $GREP "Cannot find device"`
    if [ -z "$NODEV" ]; then
	if [ -z "$QDISC_RSP" ] ; then
	    echo SUCCESS $DEVICE 
	    exit 1
	fi
    else
	echo ERROR $DEVICE not found
	exit 1
    fi

    DEV_LIST=$DEVICE
else

    for i in 0 1 2 3 4 5 6 7
    do
	DEVICE=eth$i

	QDISC_RSP=`$TC qdisc show dev $DEVICE 2>&1`
	NODEV=`$ECHO $QDISC_RSP | $GREP "Cannot find device"`
	if [ -z "$NODEV" ]; then
	    OLD_LIST="$DEV_LIST"
	    DEV_LIST="$OLD_LIST $DEVICE"
	fi
    done
fi

for i in $DEV_LIST 
do
    $TC qdisc show dev $i
    $TC class show dev $i
    $TC filter show dev $i
    echo SUCCESS $i
done

exit





